/* $Id: ar_create.c,v 0.1 1997/10/14 18:23:03 rsmit06 Exp rsmit06 $
 *
 * This file is part of the array library (libar).
 * It implements the ar_create function.
 *
 * Copyright (C) 1997  R.F. Smith
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * $Log: ar_create.c,v $
 * Revision 0.1  1997/10/14 18:23:03  rsmit06
 * Initial code
 *
 */

#include <stdlib.h>
#include "arrayp.h"

/* 
 * Creates an array. The array initially reserves space for `num'
 * elements of `size' bytes each. If the array is full, it tries to grow
 * by `num_grow' elements. If `ar', `size', `num', or `grow' are 0, 
 * the program will quit with an invalid parameter error!
 */
array_p 
ar_create(unsigned size, unsigned num, unsigned grow)
{
  array_p ar = 0;

  /* bail out on invalid parameter */
  if (num == 0 || size == 0 || grow == 0)
    ar_error("invalid parameter passed to ar_create");

  /* allocate array structure */
  ar = calloc(1, sizeof(struct _array));
  if (ar == 0)
    ar_error("memory allocation failed in ar_create");

  /* save array parameters in array_t structure */
  ar->cb_size = size;
  ar->i_max = num-1;
  ar->i_free = 0;
  ar->num_grow = num;

  /* allocate memory for array */
  ar->data = calloc(num, size);
  if (ar->data == 0)
    ar_error("memory allocation failed in ar_create");

  /* return array pointer */
  return ar;
}
