/* $Id: ar_read.c,v 0.1 1997/10/14 18:23:03 rsmit06 Exp rsmit06 $
 *
 * This file is part of the array library (libar).
 * It implements the ar_read function.
 *
 * Copyright (C) 1997  R.F. Smith
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * $Log: ar_read.c,v $
 * Revision 0.1  1997/10/14 18:23:03  rsmit06
 * Initial code
 *
 */

#include <string.h>
#include "arrayp.h"

/* 
 * Reads `count' elements form the array `ar', starting at index
 * `index'. It copies the data to `data', if `data' is not 0. If data = 0,
 * the number of readable elements is returned, but nothing is read.
 * `data' is assumed to have enough space available to hold the
 * information. It returns the actual number of elements read.
 */
int 
ar_read(array_p ar, int index, int count, void *data)
{
  unsigned offset;
  char *pch;

  /* check parameters */
  if (ar == 0 || index < 0 || count < 0)
    ar_error("invalid parameter to ar_read");

  /* no data should/can be read */
  if (count < 1 || index >= ar->i_max)
    return 0;

  /* adjust count if end of read goes past array boundary */
  if ((index+count) > ar->i_free)
    count = ar->i_free - index;

  /* copy data */
  pch = ar->data;
  offset = ar->cb_size * index;
  pch += offset;
  if (data)
    memcpy (data, pch, count * ar->cb_size);

  return count;
}
