/* $Id: ar_write.c,v 0.1 1997/10/14 18:23:03 rsmit06 Exp rsmit06 $
 *
 * This file is part of the array library (libar).
 * It implements the ar_ function.
 *
 * Copyright (C) 1997  R.F. Smith
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * $Log: ar_write.c,v $
 * Revision 0.1  1997/10/14 18:23:03  rsmit06
 * Initial code
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "arrayp.h"

/* grows the array */
extern void 
ar_grow(array_p ar);

/*
 * Writes `count' items into the array `ar'. Appends to the array if 
 * `index' = -1.  A smaller value produces an error! If `count' or 
 * `data' is 0, nothing is written. The array is expanded as needed.
 */
void 
ar_write(array_p ar, int index, int count, void *data)
{
  unsigned new_free;
  /* check parameters */
  if (ar == 0 || index < -1)
    ar_error("invalid parameter passed to ar_write");

  /* nothing to write */
  if (count == 0|| data == 0)
    return;
  
  /* prepare to append to array */
  if (index == -1) {
    index = ar->i_free;
  }

  /* expand the array if write attempt extends array bounds */
  while ((index + count) > (ar->i_max+1))
    ar_grow(ar);

  /* copy data into array */
  memcpy(((char*)ar->data)+(ar->cb_size*index), 
         data, count*ar->cb_size);

  /* change i_free if necessary */
  new_free = index+count;
  if (new_free > ar->i_free)
    ar->i_free = new_free;
}
