/* $Id: array.h,v 0.2 1997/12/07 11:28:01 rsmit06 Exp rsmit06 $
 *
 * Header file for array library (libar).
 *
 * Copyright (C) 1997  R.F. Smith
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * Read the manual page for this library with `man 3 libar'.
 *
 * $Log: array.h,v $
 * Revision 0.2  1997/12/07 11:28:01  rsmit06
 * Added `ar_rm' function.
 *
 * Revision 0.1  1997/10/14 18:23:17  rsmit06
 * Initial code
 *
 */

#ifndef ARRAY_H
#define ARRAY_H

/* The internals of this structure are private to the library. */
typedef struct _array *array_p; /* array pointer type */

/* 
 * Creates an array. The array initially reserves space for `num'
 * elements of `size' bytes each. If the array is full, it tries to grow
 * by `num_grow' elements. If `size', `num', or `grow' are 0, 
 * the program will quit with an invalid parameter error!
 */
extern array_p 
ar_create(unsigned size, unsigned num, unsigned grow);

/*
 * Destroys an array.
 */
extern void 
ar_destroy(array_p ar);

/* 
 * Reads `count' elements form the array `ar', starting at index
 * `index'. It copies the data to `data', if `data' is not 0. If data = 0,
 * the number of readable elements is returned, but nothing is read.
 * `data' is assumed to have enough space available to hold the
 * information. It returns the actual number of elements read.
 */
extern int 
ar_read(array_p ar, int index, int count, void *data);

/*
 * Writes `count' items into the array `ar'. Appends to the array if 
 * `index' == -1.  A smaller value produces an error! If `count' or 
 * `data' is 0, nothing is written. The array is expanded as needed.
 */
extern void 
ar_write(array_p ar, int index, int count, void *data);
/* use this value to append to the array */
#define AR_APPEND -1

/*
 * Returns the size of the array `ar', i.e. the number of items in use.
 */
extern int 
ar_size(array_p ar);

/* not implemented yet */
/*int ar_search(array_p ar, int (*cmp)(void*,void*), void *data);*/

/* 
 * Removes the element at index `index' from the array. Reshuffles 
 * the array to fill the gap.
 */ 
extern void 
ar_rm(array_p ar, int index);

#endif /* ARRAY_H */
